"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("../common");
const Utils_1 = require("../common/Utils");
const Types_1 = require("../livesync/Types");
const Nexus_1 = require("./Nexus");
const Types_2 = require("./Types");
let initialized = false;
const seal = Symbol();
function lock(obj, key) {
    const guard = Object.freeze({ key });
    function unlock(suppliedKey) {
        const keyRef = suppliedKey;
        if (typeof keyRef === 'symbol' && guard.key === keyRef) {
            return obj;
        }
        throw new Error('Illegal access to a protected object.');
    }
    Object.defineProperty(unlock, 'name', { value: Symbol() });
    Object.defineProperty(unlock, 'prototype', { value: null });
    Object.defineProperty(unlock, '__proto__', { value: null });
    return Object.freeze(unlock);
}
function sealModule(module) {
    return lock(module, seal);
}
function init(settings) {
    return __awaiter(this, void 0, void 0, function* () {
        if (initialized) {
            return;
        }
        const httpHeaders = {};
        const libModuleMap = new Map();
        const nexus = yield Nexus_1.default(system, settings.nexus, settings.auth);
        const liveSyncConnectionOptions = settings.livesync ? Object.freeze(Object.assign({}, settings.livesync)) : null;
        function getHttpHeaders() {
            return Object.assign({}, httpHeaders);
        }
        function setHttpHeader(key, value) {
            httpHeaders[key] = value;
        }
        function getLibModule(type) {
            const libModule = libModuleMap.get(type);
            if (!libModule) {
                throw new Error('An attempt was mad to access an instance for a missing Lib Module.');
            }
            return libModule(seal);
        }
        function liveSyncOptions() {
            if (!liveSyncConnectionOptions) {
                return null;
            }
            if (liveSyncConnectionOptions.type === common_1.ConnectionType.ORGANIZATION) {
                return { module_name: Types_1.LiveSyncConnectionType.ORGANIZATION, module_id: nexus.getUser().organization[0] };
            }
            return null;
        }
        libModuleMap.set(Types_2.LibModule.AUTH, require('../auth/Auth').default);
        libModuleMap.set(Types_2.LibModule.API, require('../api/Api').default);
        libModuleMap.set(Types_2.LibModule.LIVESYNC, require('../livesync/LiveSync').default);
        libModuleMap.set(Types_2.LibModule.MODULE, require('../module/Module').default);
        const instanceMethods = { getHttpHeaders, setHttpHeader, getLibModule, liveSyncOptions, nexus };
        Object.assign(instance, instanceMethods);
        Object.freeze(instance);
        if (settings.auth.afterConnect) {
            settings.auth.afterConnect(instance);
        }
        initialized = true;
    });
}
const [instance, system] = Utils_1.proxyWrap({}, { init, sealModule });
exports.default = system;
